package jp.co.kayo.android.localplayer.ds.dropbox;

import java.util.Calendar;

import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.VideoMedia;
import jp.co.kayo.android.localplayer.consts.TableConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioArtist;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.secret.Keys;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.provider.BaseColumns;
import android.util.Log;
import android.widget.Toast;

import com.dropbox.client2.DropboxAPI;
import com.dropbox.client2.DropboxAPI.Account;
import com.dropbox.client2.DropboxAPI.DropboxLink;
import com.dropbox.client2.DropboxAPI.Entry;
import com.dropbox.client2.android.AndroidAuthSession;
import com.dropbox.client2.android.AuthActivity;
import com.dropbox.client2.exception.DropboxException;
import com.dropbox.client2.session.AccessTokenPair;
import com.dropbox.client2.session.AppKeyPair;
import com.dropbox.client2.session.Session.AccessType;

public class DropboxHelper {
    final static private AccessType ACCESS_TYPE = AccessType.DROPBOX;

    final static private String ACCOUNT_PREFS_NAME = "prefs";
    final static private String ACCESS_KEY_NAME = "ACCESS_KEY";
    final static private String ACCESS_SECRET_NAME = "ACCESS_SECRET";
    
    public static final String AUTHORITY = "jp.co.kayo.android.localplayer";
    public static final String CONTENT_AUTHORITY_SLASH = "content://" + AUTHORITY + "/";
    public static final Uri MEDIA_CONTENT_URI = Uri.parse(CONTENT_AUTHORITY_SLASH + "audio/media");
    public static final Uri ALBUM_CONTENT_URI = Uri.parse(CONTENT_AUTHORITY_SLASH + "audio/albums");
    public static final Uri ARTIST_CONTENT_URI = Uri.parse(CONTENT_AUTHORITY_SLASH + "audio/artist");
    
    private Context context;
    private DropboxAPI<AndroidAuthSession> mApi;
    
    public DropboxHelper(Context context){
        this.context = context;
        AndroidAuthSession session = buildSession();
        mApi = new DropboxAPI<AndroidAuthSession>(session);
    }
    
    public Context getContext(){
    	return context;
    }
    
    public boolean isLoggdIn(){
        return mApi.getSession().isLinked();
    }
    
    public void startAuthentication(){
        mApi.getSession().startAuthentication(context);
    }


    public AndroidAuthSession getSession() {
        return mApi.getSession();
    }
    
    public DropboxLink media(String path) throws DropboxException{
        return mApi.media(path, false);
    }
    
    public void storeKeys(String key, String secret) {
        SharedPreferences prefs = context.getSharedPreferences(ACCOUNT_PREFS_NAME, 0);
        Editor edit = prefs.edit();
        edit.putString(ACCESS_KEY_NAME, key);
        edit.putString(ACCESS_SECRET_NAME, secret);
        edit.commit();
    }
    
    public void clearKeys() {
        SharedPreferences prefs = context.getSharedPreferences(ACCOUNT_PREFS_NAME, 0);
        Editor edit = prefs.edit();
        edit.clear();
        edit.commit();
    }
    
    private boolean checkAppKeySetup() {
        // Check to make sure that we have a valid app key
        if (Keys.APP_KEY.startsWith("CHANGE") || Keys.APP_SECRET.startsWith("CHANGE")) {
            showToast("You must apply for an app key and secret from developers.dropbox.com, and add them to the DBRoulette ap before trying it.");
            return false;
        }

        // Check if the app has set up its manifest properly.
        Intent testIntent = new Intent(Intent.ACTION_VIEW);
        String scheme = "db-" + Keys.APP_KEY;
        String uri = scheme + "://" + AuthActivity.AUTH_VERSION + "/test";
        testIntent.setData(Uri.parse(uri));
        PackageManager pm = context.getPackageManager();
        if (0 == pm.queryIntentActivities(testIntent, 0).size()) {
            showToast("URL scheme in your app's "
                    + "manifest is not set up correctly. You should have a "
                    + "com.dropbox.client2.android.AuthActivity with the " + "scheme: " + scheme);
            return false;
        }
        
        return true;
    }
    
    private void showToast(String msg) {
        Logger.d(msg);
        //Toast error = Toast.makeText(context, msg, Toast.LENGTH_LONG);
        //error.show();
    }
    
    private AndroidAuthSession buildSession() {
        AppKeyPair appKeyPair = new AppKeyPair(Keys.APP_KEY, Keys.APP_SECRET);
        AndroidAuthSession session;

        String[] stored = getKeys();
        if (stored != null) {
            AccessTokenPair accessToken = new AccessTokenPair(stored[0], stored[1]);
            session = new AndroidAuthSession(appKeyPair, ACCESS_TYPE, accessToken);
        } else {
            session = new AndroidAuthSession(appKeyPair, ACCESS_TYPE);
        }

        return session;
    }
    
    private String[] getKeys() {
        SharedPreferences prefs = context.getSharedPreferences(ACCOUNT_PREFS_NAME, 0);
        String key = prefs.getString(ACCESS_KEY_NAME, null);
        String secret = prefs.getString(ACCESS_SECRET_NAME, null);
        if (key != null && secret != null) {
            String[] ret = new String[2];
            ret[0] = key;
            ret[1] = secret;
            return ret;
        } else {
            return null;
        }
    }
    
    public Entry metadata(String directory, int limit, String s1, boolean b,
            String s2) throws DropboxException {
        return mApi.metadata(directory, limit, s1, b, s2);
    }
    
    public Account getAccountInfo() throws DropboxException{
        return mApi.accountInfo();
    }
    
    public Uri registMetaInfo(Context context, FileMetaInfo metainf){
        boolean ret;
        boolean hasRegist = isRegisterdSong(context, (metainf.path+"/"+metainf.name).hashCode());
        if(!hasRegist){
            Logger.d("metainf.path="+metainf.path);
            Logger.d("metainf.name="+metainf.name);
            Logger.d("metainf.title="+metainf.title);
            Logger.d("metainf.album="+metainf.album);
            Logger.d("metainf.artist="+metainf.artist);
            
            //アルバム情報の登録
            ret = isRegisterdAlbum(context, metainf.path.hashCode());
            if(!ret){
                Uri uri = insertAlbum(context, metainf);
                getContext().getContentResolver().notifyChange(ALBUM_CONTENT_URI, null);
            }
            //アーティスト情報の登録
            if(metainf.artist!=null && metainf.artist.length()>0){
                ret = isRegisterdArtist(context, metainf.artist.hashCode());
                if(!ret){
                    Uri uri = insertArtist(context, metainf);
                    getContext().getContentResolver().notifyChange(ARTIST_CONTENT_URI, null);
                }
            }
            //楽曲情報の登録
            Uri uri = insertSong(context, metainf);
            getContext().getContentResolver().notifyChange(MEDIA_CONTENT_URI, null);
        }
        return null;
    }
    
    public Uri registVideoInfo(Context context, VideoMetaInfo metainf){
        boolean ret;
        //楽曲情報の登録
        ret = isRegisterdVideo(context, metainf.path.hashCode());
        if(!ret){
            return insertVideo(context, metainf);
        }
        return null;
    }
    
    public boolean isRegisterdAlbum(Context context, long id){
        //有無をチェック
        Cursor cur = null;
        try{
            cur = context.getContentResolver().query(
                    MediaConsts.ALBUM_CONTENT_URI,
                    new String[]{BaseColumns._ID, AudioAlbum.ALBUM}, 
                    AudioAlbum.ALBUM_KEY + " = ?", 
                    new String[]{Long.toString(id)}, null);
            if(cur!=null && cur.moveToFirst()){
                return true;
            }
            return false;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public boolean isRegisterdArtist(Context context, long id){
        //有無をチェック
        Cursor cur = null;
        try{
            cur = context.getContentResolver().query(
                    MediaConsts.ARTIST_CONTENT_URI, 
                    new String[]{BaseColumns._ID, AudioArtist.ARTIST}, 
                    AudioArtist.ARTIST_KEY + " = ?", 
                    new String[]{Long.toString(id)}, null);
            if(cur!=null && cur.moveToFirst()){
                return true;
            }
            return false;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public boolean isRegisterdSong(Context context, long id){
        //有無をチェック
        Cursor cur = null;
        try{
            cur = context.getContentResolver().query(
                    MediaConsts.MEDIA_CONTENT_URI, 
                    new String[]{AudioMedia.MEDIA_KEY}, 
                    AudioMedia.MEDIA_KEY + " = ?", 
                    new String[]{Long.toString(id)}, null);
            if(cur!=null && cur.moveToFirst()){
                return true;
            }
            return false;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public boolean isRegisterdVideo(Context context, long id){
        //有無をチェック
        Cursor cur = null;
        try{
            cur = context.getContentResolver().query(
                    MediaConsts.VIDEO_CONTENT_URI, 
                    new String[]{AudioMedia.MEDIA_KEY}, 
                    VideoMedia.MEDIA_KEY + " = ?", 
                    new String[]{Long.toString(id)}, null);
            if(cur!=null && cur.moveToFirst()){
                return true;
            }
            return false;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public Uri insertAlbum(Context context, FileMetaInfo metainf){
        Calendar cal = Calendar.getInstance();
        Cursor cur = null;
        try{
            ContentValues values = new ContentValues();
            values.put(AudioAlbum.ALBUM_KEY, metainf.path.hashCode());
            if(metainf.album!=null && metainf.album.length()>0){
                values.put(AudioAlbum.ALBUM, metainf.album);
            }
            else{
                values.put(AudioAlbum.ALBUM, "unknown");
            }
            values.put(AudioAlbum.ALBUM_ART, metainf.albumArt);
            if(metainf.artist!=null && metainf.artist.length()>0){
                values.put(AudioAlbum.ARTIST, metainf.artist);
            }
            values.put(AudioAlbum.FIRST_YEAR, "");
            values.put(AudioAlbum.LAST_YEAR, "");
            values.put(AudioAlbum.NUMBER_OF_SONGS, "0");
            values.put(AudioAlbum.DATE_ADDED, cal.getTimeInMillis());
            values.put(AudioAlbum.DATE_MODIFIED, cal.getTimeInMillis());
            values.put(TableConsts.ALBUM_DEL_FLG, 0);
            
            return context.getContentResolver().insert(MediaConsts.ALBUM_CONTENT_URI, values);
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public Uri insertArtist(Context context, FileMetaInfo metainf){
        Calendar cal = Calendar.getInstance();
        Cursor cur = null;
        try{
            ContentValues values = new ContentValues();
            values.put(AudioArtist.ARTIST_KEY, metainf.artist.hashCode());
            values.put(AudioArtist.ARTIST, metainf.artist);
            values.put(AudioArtist.NUMBER_OF_ALBUMS, "0");
            values.put(AudioArtist.NUMBER_OF_TRACKS, "0");
            values.put(AudioArtist.DATE_ADDED, cal.getTimeInMillis());
            values.put(AudioArtist.DATE_MODIFIED, cal.getTimeInMillis());
            values.put(TableConsts.ARTIST_DEL_FLG, 0);
            
            return context.getContentResolver().insert(MediaConsts.ARTIST_CONTENT_URI, values);
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public Uri insertSong(Context context, FileMetaInfo metainf) {
        Calendar cal = Calendar.getInstance();
        Cursor cur = null;
        try{
            ContentValues values = new ContentValues();
            
            values.put(AudioMedia.MEDIA_KEY, (metainf.path+"/"+metainf.name).hashCode());
            values.put(AudioMedia.TITLE, metainf.title);
            values.put(AudioMedia.TITLE_KEY, metainf.title.hashCode());
            values.put(AudioMedia.DURATION, 0);
            if(metainf.artist!=null && metainf.artist.length()>0){
                values.put(AudioMedia.ARTIST, metainf.artist);
                values.put(AudioMedia.ARTIST_KEY, metainf.artist.hashCode());
            }
            if(metainf.album!=null && metainf.album.length()>0){
                values.put(AudioMedia.ALBUM, metainf.album);
            }
            else{
                values.put(AudioMedia.ALBUM, "unknown");
            }
            values.put(AudioMedia.ALBUM_KEY, metainf.path.hashCode());
            //values.put(AudioMedia.ALBUM_ART, "");
            values.put(AudioMedia.DATA, metainf.filepath);
            if(metainf.track == null || metainf.track.length() == 0){
                values.put(AudioMedia.TRACK, "9999");
            }
            else{
                values.put(AudioMedia.TRACK, metainf.track);
            }
            values.put(AudioMedia.DATE_ADDED, cal.getTimeInMillis());
            values.put(AudioMedia.DATE_MODIFIED, cal.getTimeInMillis());
            values.put(AudioMedia.YEAR, metainf.year);
            if(metainf.cacheFile!=null){
                values.put(TableConsts.AUDIO_CACHE_FILE, metainf.cacheFile);
            }
            
            Uri uri = context.getContentResolver().insert(MediaConsts.MEDIA_CONTENT_URI, values);
            
            return uri;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }
    
    public Uri insertVideo(Context context, VideoMetaInfo metainf) {
        Calendar cal = Calendar.getInstance();
        Cursor cur = null;
        try{
            ContentValues values = new ContentValues();
            values.put(VideoMedia.MEDIA_KEY, metainf.path.hashCode());
            values.put(VideoMedia.TITLE, metainf.title);
            values.put(VideoMedia.MIME_TYPE, metainf.mime);
            values.put(VideoMedia.RESOLUTION, "");
            values.put(VideoMedia.SIZE, metainf.size);
            values.put(VideoMedia.DATA, metainf.filepath);
            values.put(VideoMedia.DATE_ADDED, cal.getTimeInMillis());
            values.put(VideoMedia.DATE_MODIFIED, cal.getTimeInMillis());
            values.put(TableConsts.VIDEO_DEL_FLG, 0);
            
            Uri uri = context.getContentResolver().insert(MediaConsts.VIDEO_CONTENT_URI, values);
            
            return uri;
        }
        finally{
            if(cur!=null){
                cur.close();
            }
        }
    }

    public void reset() {
        context.getContentResolver().query(
                MediaConsts.RESET_CONTENT_URI, 
                null, 
                null, 
                null, null);
    }

    public String getAuthrizedURL(String lalbum_art) {
        try {
            DropboxLink link = mApi.media(lalbum_art, false);
            return link.url;
        } catch (DropboxException e) {
            Logger.e("media url cant not get", e);
        }
        return null;
    }
}
